/*
  sample caller for Minalpher v1.1 reference code version 1.1.0

Copyright (c) 2015
NTT (Nippon Telegraph and Telephone Corporation)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include<stdio.h>
#include"minalpher.h"


void hexdump(unsigned long long len, unsigned char *b, char *msg){
  unsigned long long i;

  printf(msg);
  for(i=0; i<len; i++){
    printf(" %02x", b[i]);
  }
  printf("\n");
}



int main(){
  unsigned char K[16] = {
    0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
    0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10
  };
  unsigned char N[13];
  unsigned char AD[BUFSIZ], M[BUFSIZ];
  unsigned char C[BUFSIZ + 32], tag[16];
  unsigned long long adlen, mlen, clen;
  int i, j;

  /* 16 trials */
  for(i=0; i<16; i++){
    /* setup nonce */
    for(j=0; j<13; j++){
      N[j] = i + j;
    }
    /* choose associated data and message */
    for(j=0; j<BUFSIZ; j++){
      AD[j] = (i + 3*j) % 256;
      M[j] = (i + 5*j) % 256;
    }
    /* call encryption */
    adlen = (i / 2) * (BUFSIZ / 16);
    mlen = i * (BUFSIZ / 16);
    hexdump(adlen, AD, "AD:");
    hexdump(mlen, M, "M :");
    AEAD_ENC(K, N, AD, adlen, M, mlen, C, &clen, tag);
    hexdump(clen, C, "C :");
    hexdump(16, tag, "tg:");
    /* call decryption */
    if(AEAD_DEC(K, N, AD, adlen, C, clen, tag, M, &clen)){
      printf("invalid tag\n");
    }
    /* call decryption with forged message */
    if(clen > 0){
      C[0] = 7 * i;
    }
    if(AEAD_DEC(K, N, AD, adlen, C, clen, tag, M, &clen)){
      printf("invalid tag with forged message\n");
    }
  }

  return 0;
}
